//
//  HistoryEntrySQLite.m
//  Calculator
//
//  Created by Kevin Johnson on 7/31/11.
//  Copyright 2011 Home. All rights reserved.
//

#import "CalculatorMemory.h"
#import "CalculatorEntry.h"

@implementation CalculatorMemory

@synthesize calculatorEntries;

static sqlite3 *database;
static sqlite3_stmt *deleteStmt;
static sqlite3_stmt *addStmt;


-(void) checkAndCreateDatabase{
	// Check if the SQL database has already been saved to the users phone, if not then copy it over
	BOOL success;
	
	// Create a FileManager object, we will use this to check the status
	// of the database and to copy it over if required
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	// Check if the database has already been created in the users filesystem
	success = [fileManager fileExistsAtPath:databasePath];
	
	// If the database already exists then return without doing anything
	if(success) return;
	
	// If not then proceed to copy the database from the application to the users filesystem
	
	// Get the path to the database in the application package
	NSString *databasePathFromApp = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:databaseName];
	
	// Copy the database from the package to the users filesystem
	[fileManager copyItemAtPath:databasePathFromApp toPath:databasePath error:nil];
	
	[fileManager release];
}

-(void) openDatabase {
    
    // Open the database from the users filessytem
    if(sqlite3_open([databasePath UTF8String], &database) != SQLITE_OK)
        NSAssert1(0, @"Error while trying to open database. '%s'", sqlite3_errmsg(database));
}


-(id) init {
    
    self = [super init];
    if (self) {
        // Initialization code here.
        
        // Setup some globals
        databaseName = @"CalculatorSQLite.sqlite";
        
        // Get the path to the documents directory and append the databaseName
        NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentsDir = [documentPaths objectAtIndex:0];
        databasePath = [[documentsDir stringByAppendingPathComponent:databaseName] copy];
        
        calculatorEntries = [[NSMutableArray alloc] init];
        
        // Execute the "checkAndCreateDatabase" function
        [self checkAndCreateDatabase];
        
        // Open database
        [self openDatabase];
        
    }
    
    return self;
}

-(void) readCalculatorEntriesFromDatabase {

    [calculatorEntries removeAllObjects];
	
    // Setup the SQL Statement and compile it for faster access
    const char *sqlStatement = "select * from CalculatorMemory";
    sqlite3_stmt *compiledStatement;
    if(sqlite3_prepare_v2(database, sqlStatement, -1, &compiledStatement, NULL) == SQLITE_OK) {
        // Loop through the results and add them to the feeds array
        while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
            // Read the data from the result row
            NSString *aName = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 1)];
            
            // Create a new CalculatorEntry object with the data from the database
            CalculatorEntry *calculatorEntry = [[CalculatorEntry alloc] initWithEntry:aName];
            
            // Add the CalculatorEntry object to the calculatorEntries Array
            [calculatorEntries addObject:calculatorEntry];
            
            [calculatorEntry release];
        }
    }
    // Release the compiled statement from memory
    sqlite3_finalize(compiledStatement);
}

- (NSString *) recallMemory {
    [self readCalculatorEntriesFromDatabase];
    
    return [[calculatorEntries objectAtIndex:0] entry];
}

- (void) deleteCalculatorEntry:(NSInteger)idNumber {
    
    if(deleteStmt == nil) {
        const char *sql = "delete from CalculatorMemory where id = ?";
        if(sqlite3_prepare_v2(database, sql, -1, &deleteStmt, NULL) != SQLITE_OK)
            NSAssert1(0, @"Error while creating delete statement. '%s'", sqlite3_errmsg(database));
    }
    
    //When binding parameters, index starts from 1 and not zero.
    sqlite3_bind_int(deleteStmt, 1, idNumber);
    
    if (SQLITE_DONE != sqlite3_step(deleteStmt))
        NSAssert1(0, @"Error while deleting. '%s'", sqlite3_errmsg(database));
    
    sqlite3_reset(deleteStmt);
}

- (void) clearMemory {
    [self deleteCalculatorEntry:1];
}

- (void) addCalculatorEntry:(NSString *)calculatorEntryString {
    // This method only allows 1 CalculatorEntry to be saved into the database at a time
    
    [self deleteCalculatorEntry:1]; // Delete the entry with id=1
    
    //if(addStmt == nil) {
        const char *sql = "insert into CalculatorMemory(id, entry) Values(?, ?)";
        if(sqlite3_prepare_v2(database, sql, -1, &addStmt, NULL) != SQLITE_OK)
            NSAssert1(0, @"Error while creating add statement. '%s'", sqlite3_errmsg(database));
    //}
    
    sqlite3_bind_int(addStmt, 1, 1); // Adds new entry with id=1
    sqlite3_bind_text(addStmt, 2, [calculatorEntryString UTF8String], -1, SQLITE_TRANSIENT);
    
    if(SQLITE_DONE != sqlite3_step(addStmt))
        NSAssert1(0, @"Error while inserting data. '%s'", sqlite3_errmsg(database));
    //else
        //SQLite provides a method to get the last primary key inserted by using sqlite3_last_insert_rowid
        //id = sqlite3_last_insert_rowid(database);
    
    //Reset the add statement.
    sqlite3_reset(addStmt);
}

- (void) finalizeStatements {
    
    if(database) sqlite3_close(database);
    if(deleteStmt) sqlite3_finalize(deleteStmt);
    if(addStmt) sqlite3_finalize(addStmt);
}
 

- (void) dealloc {
    [databaseName release];
    [databasePath release];
    [self finalizeStatements];
    [calculatorEntries release];
	[super dealloc];
}


@end
